@@ -6,6 +6,7 @@ import (
 	"errors"
 	"fmt"
 	"image"
+	"io/fs"
 	"log/slog"
 	"os"
 	"os/exec"
@@ -16,7 +17,11 @@ import (
 	"strings"
 	"time"
 
+	"golang.org/x/text/encoding/unicode"
+	"golang.org/x/text/transform"
+
 	"github.com/yorukot/superfile/src/internal/ui"
+	"github.com/yorukot/superfile/src/internal/ui/rendering"
 
 	"github.com/yorukot/superfile/src/internal/common"
 	"github.com/yorukot/superfile/src/internal/utils"
@@ -26,7 +31,6 @@ import (
 	"github.com/charmbracelet/x/exp/term/ansi"
 	"github.com/yorukot/ansichroma"
 	"github.com/yorukot/superfile/src/config/icon"
-	filepreview "github.com/yorukot/superfile/src/pkg/file_preview"
 )
 
 func (m *model) sidebarRender() string {
@@ -40,7 +44,7 @@ func (m *model) filePanelRender() string {
 	f := make([]string, len(m.fileModel.filePanels))
 	for i, filePanel := range m.fileModel.filePanels {
 		// check if cursor or render out of range
-		// Todo - instead of this, have a filepanel.validateAndFix(), and log Error
+		// TODO - instead of this, have a filepanel.validateAndFix(), and log Error
 		// This should not ever happen
 		if filePanel.cursor > len(filePanel.element)-1 {
 			filePanel.cursor = 0
@@ -48,7 +52,7 @@ func (m *model) filePanelRender() string {
 		}
 		m.fileModel.filePanels[i] = filePanel
 
-		// Todo : Move this to a utility function and clarify the calculation via comments
+		// TODO : Move this to a utility function and clarify the calculation via comments
 		// Maybe even write unit tests
 		var filePanelWidth int
 		if (m.fullWidth-common.Config.SidebarWidth-(4+(len(m.fileModel.filePanels)-1)*2))%len(m.fileModel.filePanels) != 0 && i == len(m.fileModel.filePanels)-1 {
@@ -69,93 +73,124 @@ func (m *model) filePanelRender() string {
 func (panel *filePanel) Render(mainPanelHeight int, filePanelWidth int, focussed bool) string {
 	r := ui.FilePanelRenderer(mainPanelHeight+2, filePanelWidth+2, focussed)
 
-	// Todo - Add ansitruncate left in renderer and remove truncation here
-	r.AddLines(common.FilePanelTopDirectoryIcon + common.FilePanelTopPathStyle.Render(
-		common.TruncateTextBeginning(panel.location, filePanelWidth-4, "...")))
+	panel.renderTopBar(r, filePanelWidth)
+	panel.renderSearchBar(r)
+	panel.renderFooter(r)
+	panel.renderFileEntries(r, mainPanelHeight, filePanelWidth)
 
-	// Todo : Unit test all these if else chains (?)
-	// Todo : Better move it out to a separate function
-	var sortTypeString string
-	var sortTypeStringSmall string
-	if panel.sortOptions.data.reversed {
-		sortTypeStringSmall = icon.SortDesc
-	} else {
-		sortTypeStringSmall = icon.SortAsc
-	}
-
-	// Todo : Make "Date Modified" a constant, and move this to a utility function
-	if panel.sortOptions.data.options[panel.sortOptions.data.selected] == "Date Modified" {
-		sortTypeString = "Date"
-	} else {
-		sortTypeString = panel.sortOptions.data.options[panel.sortOptions.data.selected]
-	}
-
-	if common.Config.Nerdfont {
-		sortTypeString = sortTypeStringSmall + icon.Space + sortTypeString
-	} else {
-		// Todo : Figure out if we can set icon.Space to " " if nerdfont is false
-		sortTypeString = sortTypeStringSmall + " " + sortTypeString
-	}
-
-	var panelModeString string
-	var panelModeStringSmall string
-
-	if panel.panelMode == browserMode {
-		panelModeStringSmall = icon.Browser
-		panelModeString = "Browser"
-	} else if panel.panelMode == selectMode {
-		panelModeStringSmall = icon.Select
-		panelModeString = "Select"
-	}
-
-	// Only append Icon in case of nerdfont being true
-	if common.Config.Nerdfont {
-		panelModeString = panelModeStringSmall + icon.Space + panelModeString
-	}
+	return r.Render()
+}
 
+func (panel *filePanel) renderTopBar(r *rendering.Renderer, filePanelWidth int) {
+	// TODO - Add ansitruncate left in renderer and remove truncation here
+	truncatedPath := common.TruncateTextBeginning(panel.location, filePanelWidth-4, "...")
+	r.AddLines(common.FilePanelTopDirectoryIcon + common.FilePanelTopPathStyle.Render(truncatedPath))
 	r.AddSection()
+}
+
+func (panel *filePanel) renderSearchBar(r *rendering.Renderer) {
 	r.AddLines(" " + panel.searchBar.View())
+}
 
-	cursorNumber := panel.cursor
+// TODO : Unit test this
+func (panel *filePanel) renderFooter(r *rendering.Renderer) {
+	sortLabel, sortIcon := panel.getSortInfo()
+	modeLabel, modeIcon := panel.getPanelModeInfo()
+	cursorStr := panel.getCursorString()
 
-	// Make 1-indexed only for non zero filePanel len
-	if len(panel.element) > 0 {
-		cursorNumber++
+	if common.Config.Nerdfont {
+		sortLabel = sortIcon + icon.Space + sortLabel
+		modeLabel = modeIcon + icon.Space + modeLabel
+	} else {
+		// TODO : Figure out if we can set icon.Space to " " if nerdfont is false
+		// That would simplify code
+		sortLabel = sortIcon + " " + sortLabel
 	}
-	cursorNumberString := fmt.Sprintf("%d/%d", cursorNumber, len(panel.element))
 
 	if common.Config.ShowPanelFooterInfo {
-		r.SetBorderInfoItems(sortTypeString, panelModeString, cursorNumberString)
+		r.SetBorderInfoItems(sortLabel, modeLabel, cursorStr)
 		if r.AreInfoItemsTruncated() {
-			// Use smaller values
-			r.SetBorderInfoItems(sortTypeStringSmall, panelModeStringSmall, cursorNumberString)
+			r.SetBorderInfoItems(sortIcon, modeIcon, cursorStr)
 		}
 	} else {
-		r.SetBorderInfoItems(cursorNumberString)
+		r.SetBorderInfoItems(cursorStr)
 	}
+}
 
+func (panel *filePanel) renderFileEntries(r *rendering.Renderer, mainPanelHeight, filePanelWidth int) {
 	if len(panel.element) == 0 {
 		r.AddLines(common.FilePanelNoneText)
-	} else {
-		for h := panel.render; h < panel.render+panelElementHeight(mainPanelHeight) && h < len(panel.element); h++ {
-			cursor := " "
-			// Check if the cursor needs to be displayed, if the user is using the search bar, the cursor is not displayed
-			if h == panel.cursor && !panel.searchBar.Focused() {
-				cursor = icon.Cursor
-			}
-			isItemSelected := arrayContains(panel.selected, panel.element[h].location)
-			if panel.renaming && h == panel.cursor {
-				r.AddLines(panel.rename.View())
-			} else {
-				// Todo (Performance) : Figure out why we are doing this. This will unnecessarily slow down
-				// rendering. There should be a way to avoid this at render
-				_, err := os.ReadDir(panel.element[h].location)
-				r.AddLines(common.FilePanelCursorStyle.Render(cursor+" ") + common.PrettierName(panel.element[h].name, filePanelWidth-5,
-					panel.element[h].directory || (err == nil), isItemSelected, common.FilePanelBGColor))
-			}
-		}
+		return
 	}
-	return r.Render()
+
+	end := min(panel.render+panelElementHeight(mainPanelHeight), len(panel.element))
+
+	for i := panel.render; i < end; i++ {
+		// TODO : Fix this, this is O(n^2) complexity. Considered a file panel with 200 files, and 100 selected
+		// We will be doing a search in 100 item slice for all 200 files.
+		isSelected := arrayContains(panel.selected, panel.element[i].location)
+
+		if panel.renaming && i == panel.cursor {
+			r.AddLines(panel.rename.View())
+			continue
+		}
+
+		cursor := " "
+		if i == panel.cursor && !panel.searchBar.Focused() {
+			cursor = icon.Cursor
+		}
+
+		// Performance TODO: Remove or cache this if not needed at render time
+		// This will unnecessarily slow down rendering. There should be a way to avoid this at render
+		_, err := os.ReadDir(panel.element[i].location)
+		dirExists := err == nil || panel.element[i].directory
+
+		renderedName := common.PrettierName(
+			panel.element[i].name,
+			filePanelWidth-5,
+			dirExists,
+			isSelected,
+			common.FilePanelBGColor,
+		)
+
+		r.AddLines(common.FilePanelCursorStyle.Render(cursor+" ") + renderedName)
+	}
+}
+
+// TODO : Make these strings : "Date Modified", "Date", "Browser", "Select" a constant
+func (panel *filePanel) getSortInfo() (string, string) {
+	opts := panel.sortOptions.data
+	selected := opts.options[opts.selected]
+	label := selected
+	if selected == "Date Modified" {
+		label = "Date"
+	}
+
+	iconStr := icon.SortAsc
+
+	if opts.reversed {
+		iconStr = icon.SortDesc
+	}
+	return label, iconStr
+}
+
+func (panel *filePanel) getPanelModeInfo() (string, string) {
+	switch panel.panelMode {
+	case browserMode:
+		return "Browser", icon.Browser
+	case selectMode:
+		return "Select", icon.Select
+	default:
+		return "", ""
+	}
+}
+
+func (panel *filePanel) getCursorString() string {
+	cursor := panel.cursor
+	if len(panel.element) > 0 {
+		cursor++ // Convert to 1-based
+	}
+	return fmt.Sprintf("%d/%d", cursor, len(panel.element))
 }
 
 func (m *model) processBarRender() string {
@@ -168,7 +203,7 @@ func (m *model) processBarRender() string {
 
 	// cursor's value itself cannot be used as its zero indexed
 	cursorNumber := 0
-	// Todo : Instead of directly accessing slice, there should be a method .IsEmpty() , or .CntProcess()
+	// TODO : Instead of directly accessing slice, there should be a method .IsEmpty() , or .CntProcess()
 	if len(m.processBarModel.processList) != 0 {
 		cursorNumber = m.processBarModel.cursor + 1
 	}
@@ -181,7 +216,7 @@ func (m *model) processBarRender() string {
 
 	// save process in the array and sort the process by finished or not,
 	// completion percetage, or finish time
-	// Todo : This is very inefficient and can be improved.
+	// TODO : This is very inefficient and can be improved.
 	// The whole design needs to be changed so that we dont need to recreate the slice
 	// and sort on each render. Idea : Maintain two slices - completed, ongoing
 	// Processes should be added / removed to the slice on correct time, and we dont
@@ -223,16 +258,16 @@ func (m *model) processBarRender() string {
 		curProcess := processes[i]
 		curProcess.progress.Width = utils.FooterWidth(m.fullWidth) - 3
 
-		// Todo : get them via a separate function.
+		// TODO : get them via a separate function.
 		var symbol string
 		var cursor string
 		if i == m.processBarModel.cursor {
-			// Todo : Prerender it.
+			// TODO : Prerender it.
 			cursor = common.FooterCursorStyle.Render("┃ ")
 		} else {
 			cursor = common.FooterCursorStyle.Render("  ")
 		}
-		// Todo : Prerender
+		// TODO : Prerender
 		switch curProcess.state {
 		case failure:
 			symbol = common.ProcessErrorStyle.Render(icon.Warn)
@@ -243,9 +278,19 @@ func (m *model) processBarRender() string {
 		case cancel:
 			symbol = common.ProcessCancelStyle.Render(icon.Error)
 		}
-
+		slog.Debug("process", "name", curProcess.name, "done", curProcess.done, "total", curProcess.total, "state", curProcess.state, "symbol", symbol)
 		r.AddLines(cursor + common.FooterStyle.Render(common.TruncateText(curProcess.name, utils.FooterWidth(m.fullWidth)-7, "...")+" ") + symbol)
-		r.AddLines(cursor+curProcess.progress.ViewAs(float64(curProcess.done)/float64(curProcess.total)), "")
+
+		// calculate progress percentage
+		// if the total is 0 mean the process only have directory
+		// so we can set the progress to 100%
+		if curProcess.total != 0 {
+			progressPercentage := float64(curProcess.done) / float64(curProcess.total)
+			slog.Debug("progress", "percentage", progressPercentage)
+			r.AddLines(cursor+curProcess.progress.ViewAs(progressPercentage), "")
+		} else {
+			r.AddLines(cursor + curProcess.progress.ViewAs(1))
+		}
 	}
 
 	return r.Render()
@@ -253,71 +298,113 @@ func (m *model) processBarRender() string {
 
 // This updates m.fileMetaData
 func (m *model) metadataRender() string {
-	// process bar
-	if len(m.fileMetaData.metaData) == 0 && len(m.fileModel.filePanels[m.filePanelFocusIndex].element) > 0 && !m.fileModel.renaming {
-		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{"", ""})
-		m.fileMetaData.metaData = append(m.fileMetaData.metaData, [2]string{" " + icon.InOperation + "  Loading metadata...", ""})
-		// Todo : This needs to be improved, we are updating m.fileMetaData is a separate goroutine
+	m.ensureMetadataLoaded()
+
+	// TODO : This is bad, this is bad mixing rendering of content and loading of content.
+	// The metadata should be filled in slice correctly at the time its loaded, not when we
+	// are rendering it.
+	sortMetadata(m.fileMetaData.metaData)
+	maxKeyLen := getMaxKeyLength(m.fileMetaData.metaData)
+	sprintfLen, valLen := computeMetadataWidths(m.fullWidth, maxKeyLen)
+
+	r := ui.MetadataRenderer(m.footerHeight+2, utils.FooterWidth(m.fullWidth)+2, m.focusPanel == metadataFocus)
+	if len(m.fileMetaData.metaData) > 0 {
+		r.SetBorderInfoItems(fmt.Sprintf("%d/%d", m.fileMetaData.renderIndex+1, len(m.fileMetaData.metaData)))
+	}
+
+	lines := formatMetadataLines(m.fileMetaData.metaData, m.fileMetaData.renderIndex, m.footerHeight, sprintfLen, valLen)
+	r.AddLines(lines...)
+
+	return r.Render()
+}
+
+func (m *model) ensureMetadataLoaded() {
+	if len(m.fileMetaData.metaData) == 0 &&
+		len(m.fileModel.filePanels[m.filePanelFocusIndex].element) > 0 &&
+		!m.fileModel.renaming {
+		m.fileMetaData.metaData = [][2]string{
+			{"", ""},
+			{" " + icon.InOperation + "  Loading metadata...", ""},
+		}
+		// TODO : This needs to be improved, we are updating m.fileMetaData is a separate goroutine
 		// while also modifying it here in the function. It could cause issues.
 		go func() {
 			m.returnMetaData()
 		}()
 	}
+}
 
-	// Todo : The whole intention of this is to get the comparisonFields come before
-	// other fields. Sorting like this is a bad way of achieving that. This can be improved
-	sort.Slice(m.fileMetaData.metaData, func(i, j int) bool {
-		// Initialising a new slice in each check by sort functions is too ineffinceint.
-		// Todo : Fix it
-		comparisonFields := []string{"Name", "Size", "Date Modified", "Date Accessed"}
+// TODO : Move this and many other utility function to separate files
+// and unit test them too.
+func sortMetadata(meta [][2]string) {
+	priority := map[string]int{
+		"Name":          0,
+		"Size":          1,
+		"Date Modified": 2,
+		"Date Accessed": 3,
+	}
 
-		for _, field := range comparisonFields {
-			if m.fileMetaData.metaData[i][0] == field {
-				return true
-			} else if m.fileMetaData.metaData[j][0] == field {
-				return false
-			}
+	sort.SliceStable(meta, func(i, j int) bool {
+		pi, iOkay := priority[meta[i][0]]
+		pj, jOkay := priority[meta[j][0]]
+
+		// Both are priority fields
+		if iOkay && jOkay {
+			return pi < pj
+		}
+		// i is a priority field, and j is not
+		if iOkay {
+			return true
 		}
 
-		// Default comparison
-		return m.fileMetaData.metaData[i][0] < m.fileMetaData.metaData[j][0]
+		// j is a priority field, and i is not
+		if jOkay {
+			return false
+		}
+
+		// None of them are priority fields, sort with name
+		return meta[i][0] < meta[j][0]
 	})
+}
 
-	// Part where actual rendering happens.
-	maxKeyLength := 0
-	for _, data := range m.fileMetaData.metaData {
-		if len(data[0]) > maxKeyLength {
-			maxKeyLength = len(data[0])
+func getMaxKeyLength(meta [][2]string) int {
+	maxLen := 0
+	for _, pair := range meta {
+		if len(pair[0]) > maxLen {
+			maxLen = len(pair[0])
 		}
 	}
+	return maxLen
+}
 
-	// Todo : Too much calculations that are not in a fuctions, are not
-	// unit tested, and have no proper explanation. This makes it
-	// very hard to maintain and add any changes
-	sprintfLength := maxKeyLength + 1
-	valueLength := utils.FooterWidth(m.fullWidth) - maxKeyLength - 2
-	if valueLength < utils.FooterWidth(m.fullWidth)/2 {
-		valueLength = utils.FooterWidth(m.fullWidth)/2 - 2
-		sprintfLength = valueLength
-	}
-	r := ui.MetadataRenderer(m.footerHeight+2, utils.FooterWidth(m.fullWidth)+2, m.focusPanel == metadataFocus)
-	// Todo : We can take this info as input in metadata renderer constructor
-	renderIndex := m.fileMetaData.renderIndex
-	if len(m.fileMetaData.metaData) > 0 {
-		renderIndex++
-	}
-	r.SetBorderInfoItems(fmt.Sprintf("%d/%d", renderIndex, len(m.fileMetaData.metaData)))
+func computeMetadataWidths(fullWidth, maxKeyLen int) (int, int) {
+	metadataPanelWidth := utils.FooterWidth(fullWidth)
 
-	imax := min(m.footerHeight+m.fileMetaData.renderIndex, len(m.fileMetaData.metaData))
-	for i := m.fileMetaData.renderIndex; i < imax; i++ {
-		data := common.TruncateMiddleText(m.fileMetaData.metaData[i][1], valueLength, "...")
-		metadataName := m.fileMetaData.metaData[i][0]
-		if utils.FooterWidth(m.fullWidth)-maxKeyLength-3 < utils.FooterWidth(m.fullWidth)/2 {
-			metadataName = common.TruncateMiddleText(m.fileMetaData.metaData[i][0], valueLength, "...")
+	// Value Length = PanelLength - Key length - 2 (for border)
+	valueLen := metadataPanelWidth - maxKeyLen - 2
+	sprintfLen := maxKeyLen + 1
+	if valueLen < metadataPanelWidth/2 {
+		valueLen = metadataPanelWidth/2 - 2
+		sprintfLen = valueLen
+	}
+
+	return sprintfLen, valueLen
+}
+
+// TODO : Simplify these mystic calculations, or add explanation comments.
+func formatMetadataLines(meta [][2]string, startIdx, height, sprintfLen, valueLen int) []string {
+	lines := []string{}
+	endIdx := min(startIdx+height, len(meta))
+	for i := startIdx; i < endIdx; i++ {
+		key := meta[i][0]
+		value := common.TruncateMiddleText(meta[i][1], valueLen, "...")
+		if utils.FooterWidth(0)-sprintfLen-3 < utils.FooterWidth(0)/2 {
+			key = common.TruncateMiddleText(key, valueLen, "...")
 		}
-		r.AddLines(fmt.Sprintf("%-*s %s", sprintfLength, metadataName, data))
+		line := fmt.Sprintf("%-*s %s", sprintfLen, key, value)
+		lines = append(lines, line)
 	}
-	return r.Render()
+	return lines
 }
 
 func (m *model) clipboardRender() string {
@@ -330,7 +417,7 @@ func (m *model) clipboardRender() string {
 	}
 	r := ui.ClipboardRenderer(m.footerHeight+2, bottomWidth+2)
 	if len(m.copyItems.items) == 0 {
-		// Todo move this to a string
+		// TODO move this to a string
 		r.AddLines("", " "+icon.Error+"  No content in clipboard")
 	} else {
 		for i := 0; i < len(m.copyItems.items) && i < m.footerHeight; i++ {
@@ -343,7 +430,7 @@ func (m *model) clipboardRender() string {
 					slog.Error("Clipboard render function get item state ", "error", err)
 				}
 				if !os.IsNotExist(err) {
-					// Todo : There is an inconsistency in parameter that is being passed, and its name in ClipboardPrettierName function
+					// TODO : There is an inconsistency in parameter that is being passed, and its name in ClipboardPrettierName function
 					r.AddLines(common.ClipboardPrettierName(m.copyItems.items[i], utils.FooterWidth(m.fullWidth)-3, fileInfo.IsDir(), false))
 				}
 			}
@@ -413,7 +500,12 @@ func (m *model) typineModalRender() string {
 		lipgloss.NewStyle().Background(common.ModalBGColor).Render("           ") +
 		cancel
 
-	return common.ModalBorderStyle(common.ModalHeight, common.ModalWidth).Render(fileLocation + "\n" + m.typingModal.textInput.View() + "\n\n" + tip)
+	var err string
+	if m.typingModal.errorMesssage != "" {
+		err = "\n\n" + common.ModalErrorStyle.Render(m.typingModal.errorMesssage)
+	}
+	// TODO : Move this all to rendering package to avoid specifying newlines manually
+	return common.ModalBorderStyle(common.ModalHeight, common.ModalWidth).Render(fileLocation + "\n" + m.typingModal.textInput.View() + "\n\n" + tip + err)
 }
 
 func (m *model) introduceModalRender() string {
@@ -435,6 +527,14 @@ func (m *model) warnModalRender() string {
 	return common.ModalBorderStyle(common.ModalHeight, common.ModalWidth).Render(title + "\n\n" + content + "\n\n" + tip)
 }
 
+func (m *model) notifyModalRender() string {
+	title := m.notifyModal.title
+	content := m.notifyModal.content
+	okay := common.ModalConfirm.Render(" (" + common.Hotkeys.Confirm[0] + ") Okay ")
+	okay = common.MainStyle.AlignHorizontal(lipgloss.Center).AlignVertical(lipgloss.Center).Render(okay)
+	return common.ModalBorderStyle(common.ModalHeight, common.ModalWidth).Render(title + "\n\n" + content + "\n\n" + okay)
+}
+
 func (m *model) promptModalRender() string {
 	return m.promptModal.Render()
 }
@@ -546,7 +646,8 @@ func readFileContent(filepath string, maxLineLength int, previewLine int) (strin
 	}
 	defer file.Close()
 
-	scanner := bufio.NewScanner(file)
+	reader := transform.NewReader(file, unicode.BOMOverride(unicode.UTF8.NewDecoder()))
+	scanner := bufio.NewScanner(reader)
 	lineCount := 0
 	for scanner.Scan() {
 		line := scanner.Text()
@@ -563,134 +664,124 @@ func readFileContent(filepath string, maxLineLength int, previewLine int) (strin
 }
 
 func (m *model) filePreviewPanelRender() string {
-	// Todo : This width adjustment must not be done inside render function. It should
+	// TODO : This width adjustment must not be done inside render function. It should
 	// only be triggered via Update()
 	m.fileModel.filePreview.width += m.fullWidth - common.Config.SidebarWidth - m.fileModel.filePreview.width - ((m.fileModel.width + 2) * len(m.fileModel.filePanels)) - 2
 
 	return m.filePreviewPanelRenderWithDimensions(m.mainPanelHeight+2, m.fileModel.filePreview.width)
 }
 
-func (m *model) filePreviewPanelRenderWithDimensions(previewHeight int, previewWidth int) string {
-	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
-	box := common.FilePreviewBox(previewHeight, previewWidth)
-	r := ui.FilePreviewPanelRenderer(previewHeight, previewWidth)
+// Helper function to handle empty panel case
+func (m *model) renderEmptyFilePreview(r *rendering.Renderer) string {
+	return r.Render()
+}
 
-	if len(panel.element) == 0 {
-		r.AddLines(common.FilePreviewNoContentText)
-		return r.Render()
-	}
-	// This could create errors if panel.cursor ever becomes negative, or goes out of bounds
-	// We should have a panel validation function in our View() function
-	// Panel is a full fledged object with own state, its accessed and modified so many times.
-	// Ideally we dont should never access data from it via directly accessing its variables
-	// Todo : Instead we should have helper functions for panel object and access data that way
-	// like panel.GetCurrentSelectedElem() . This abstration of implemetation of panel is needed.
-	// Now this lack of abstraction has caused issues ( See PR#730 ) . And now
-	// someone needs to scan through the entire codebase to figure out which access of panel
-	// data is causing crash.
-	itemPath := panel.element[panel.cursor].location
+// Helper function to handle file info errors
+func (m *model) renderFileInfoError(r *rendering.Renderer, _ lipgloss.Style, err error) string {
+	slog.Error("Error get file info", "error", err)
+	return r.Render()
+}
 
-	// Renamed it to info_err to prevent shadowing with err below
-	fileInfo, infoErr := os.Stat(itemPath)
+// Helper function to handle unsupported formats
+func (m *model) renderUnsupportedFormat(box lipgloss.Style) string {
+	return box.Render(common.FilePreviewUnsupportedFormatText)
+}
 
-	if infoErr != nil {
-		slog.Error("Error get file info", "error", infoErr)
-		r.AddLines(common.FilePreviewNoFileInfoText)
+// Helper function to handle unsupported mode
+func (m *model) renderUnsupportedFileMode(r *rendering.Renderer) string {
+	r.AddLines(common.FilePreviewUnsupportedFileMode)
+	return r.Render()
+}
+
+// Helper function to handle directory preview
+func (m *model) renderDirectoryPreview(r *rendering.Renderer, itemPath string, previewHeight int) string {
+	files, err := os.ReadDir(itemPath)
+	if err != nil {
+		slog.Error("Error render directory preview", "error", err)
+		r.AddLines(common.FilePreviewDirectoryUnreadableText)
 		return r.Render()
 	}
 
-	ext := filepath.Ext(itemPath)
-	// check if the file is unsupported file, cuz pdf will cause error
-	if slices.Contains(common.UnsupportedPreviewFormats, ext) {
-		r.AddLines(common.FilePreviewUnsupportedFormatText)
+	if len(files) == 0 {
+		r.AddLines(common.FilePreviewEmptyText)
 		return r.Render()
 	}
 
-	if fileInfo.IsDir() {
-		dirPath := itemPath
-
-		files, err := os.ReadDir(dirPath)
-		if err != nil {
-			slog.Error("Error render directory preview", "error", err)
-			r.AddLines(common.FilePreviewDirectoryUnreadableText)
-			return r.Render()
+	sort.Slice(files, func(i, j int) bool {
+		if files[i].IsDir() && !files[j].IsDir() {
+			return true
 		}
-
-		if len(files) == 0 {
-			r.AddLines(common.FilePreviewEmptyText)
-			return r.Render()
+		if !files[i].IsDir() && files[j].IsDir() {
+			return false
 		}
+		return files[i].Name() < files[j].Name()
+	})
 
-		sort.Slice(files, func(i, j int) bool {
-			if files[i].IsDir() && !files[j].IsDir() {
-				return true
-			}
-			if !files[i].IsDir() && files[j].IsDir() {
-				return false
-			}
-			return files[i].Name() < files[j].Name()
-		})
+	for i := 0; i < previewHeight && i < len(files); i++ {
+		file := files[i]
+		style := common.GetElementIcon(file.Name(), file.IsDir(), common.Config.Nerdfont)
+		res := lipgloss.NewStyle().Foreground(lipgloss.Color(style.Color)).Background(common.FilePanelBGColor).
+			Render(style.Icon+" ") + common.FilePanelStyle.Render(file.Name())
+		r.AddLines(res)
+	}
+	return r.Render()
+}
 
-		for i := 0; i < previewHeight && i < len(files); i++ {
-			file := files[i]
-
-			style := common.GetElementIcon(file.Name(), file.IsDir(), common.Config.Nerdfont)
-
-			res := lipgloss.NewStyle().Foreground(lipgloss.Color(style.Color)).Background(common.FilePanelBGColor).
-				Render(style.Icon+" ") + common.FilePanelStyle.Render(file.Name())
-
-			r.AddLines(res)
-		}
-		return r.Render()
+// Helper function to handle image preview
+func (m *model) renderImagePreview(box lipgloss.Style, itemPath string, previewWidth, previewHeight int, sideAreaWidth int) string {
+	if !m.fileModel.filePreview.open {
+		return box.Render("\n --- Preview panel is closed ---")
 	}
 
-	if isImageFile(itemPath) {
-		if !m.fileModel.filePreview.open {
-			// Todo : These variables can be pre rendered for efficiency and less duplicacy
-			return box.Render("\n --- Preview panel is closed ---")
-		}
-
-		if !common.Config.ShowImagePreview {
-			return box.Render("\n --- Image preview is disabled ---")
-		}
-
-		ansiRender, err := filepreview.ImagePreview(itemPath, previewWidth, previewHeight, common.Theme.FilePanelBG)
-		if errors.Is(err, image.ErrFormat) {
-			return box.Render("\n --- " + icon.Error + " Unsupported image formats ---")
-		}
-
-		if err != nil {
-			slog.Error("Error covernt image to ansi", "error", err)
-			return box.Render("\n --- " + icon.Error + " Error covernt image to ansi ---")
-		}
-
-		return box.AlignVertical(lipgloss.Center).AlignHorizontal(lipgloss.Center).Render(ansiRender)
+	if !common.Config.ShowImagePreview {
+		return box.Render("\n --- Image preview is disabled ---")
 	}
 
+	// Use the new auto-detection function to choose the best renderer
+	imageRender, err := m.imagePreviewer.ImagePreview(itemPath, previewWidth, previewHeight, common.Theme.FilePanelBG, sideAreaWidth)
+	if errors.Is(err, image.ErrFormat) {
+		return box.Render("\n --- " + icon.Error + " Unsupported image formats ---")
+	}
+
+	if err != nil {
+		slog.Error("Error convert image to ansi", "error", err)
+		return box.Render("\n --- " + icon.Error + " Error convert image to ansi ---")
+	}
+
+	// Check if this looks like Kitty protocol output (starts with escape sequences)
+	// For Kitty protocol, avoid using lipgloss alignment to prevent layout drift
+	if strings.HasPrefix(imageRender, "\x1b_G") {
+		rendered := common.FilePreviewBox(previewHeight, previewWidth).Render(imageRender)
+		return rendered
+	}
+
+	// For ANSI output, we can safely use vertical alignment
+	return box.AlignVertical(lipgloss.Center).AlignHorizontal(lipgloss.Center).Render(imageRender)
+}
+
+// Helper function to handle text file preview
+func (m *model) renderTextPreview(r *rendering.Renderer, box lipgloss.Style, itemPath string, previewWidth, previewHeight int) string {
 	format := lexers.Match(filepath.Base(itemPath))
-
 	if format == nil {
 		isText, err := common.IsTextFile(itemPath)
 		if err != nil {
 			slog.Error("Error while checking text file", "error", err)
-			return box.Render("\n --- " + icon.Error + " Error get file info ---")
+			return box.Render(common.FilePreviewError)
 		} else if !isText {
-			return box.Render("\n --- " + icon.Error + " Unsupported formats ---")
+			return box.Render(common.FilePreviewUnsupportedFormatText)
 		}
 	}
 
-	// At this point either format is not nil, or we can read the file
 	fileContent, err := readFileContent(itemPath, previewWidth, previewHeight)
 	if err != nil {
 		slog.Error("Error open file", "error", err)
-		return box.Render("\n --- " + icon.Error + " Error open file ---")
+		return box.Render(common.FilePreviewError)
 	}
 
 	if fileContent == "" {
-		return box.Render("\n --- empty ---")
+		return box.Render(common.FilePreviewEmptyText)
 	}
 
-	// We know the format of file, and we can apply syntax highlighting
 	if format != nil {
 		background := ""
 		if !common.Config.TransparentBackground {
@@ -706,13 +797,62 @@ func (m *model) filePreviewPanelRenderWithDimensions(previewHeight int, previewW
 		}
 		if err != nil {
 			slog.Error("Error render code highlight", "error", err)
-			return box.Render("\n --- " + icon.Error + " Error render code highlight ---")
+			return box.Render("\n" + common.FilePreviewError)
 		}
 	}
+
 	r.AddLines(fileContent)
 	return r.Render()
 }
 
+func (m *model) filePreviewPanelRenderWithDimensions(previewHeight int, previewWidth int) string {
+	panel := m.fileModel.filePanels[m.filePanelFocusIndex]
+	box := common.FilePreviewBox(previewHeight, previewWidth)
+	r := ui.FilePreviewPanelRenderer(previewHeight, previewWidth)
+	clearCmd := m.imagePreviewer.ClearKittyImages()
+	if len(panel.element) == 0 {
+		return m.renderEmptyFilePreview(r) + clearCmd
+	}
+
+	// This could create errors if panel.cursor ever becomes negative, or goes out of bounds
+	// We should have a panel validation function in our View() function
+	// Panel is a full fledged object with own state, its accessed and modified so many times.
+	// Ideally we dont should never access data from it via directly accessing its variables
+	// TODO : Instead we should have helper functions for panel object and access data that way
+	// like panel.GetCurrentSelectedElem() . This abstration of implemetation of panel is needed.
+	// Now this lack of abstraction has caused issues ( See PR#730 ) . And now
+	// someone needs to scan through the entire codebase to figure out which access of panel
+	// data is causing crash.
+	itemPath := panel.element[panel.cursor].location
+	fileInfo, infoErr := os.Stat(itemPath)
+	if infoErr != nil {
+		return m.renderFileInfoError(r, box, infoErr) + clearCmd
+	}
+	slog.Debug("Attempting to render preview", "itemPath", itemPath,
+		"mode", fileInfo.Mode().String(), "isRegular", fileInfo.Mode().IsRegular())
+
+	// For non regular files which are not directories Dont try to read them
+	// See Issue
+	if !fileInfo.Mode().IsRegular() && (fileInfo.Mode()&fs.ModeDir) == 0 {
+		return m.renderUnsupportedFileMode(r) + clearCmd
+	}
+
+	ext := filepath.Ext(itemPath)
+	if slices.Contains(common.UnsupportedPreviewFormats, ext) {
+		return m.renderUnsupportedFormat(box) + clearCmd
+	}
+
+	if fileInfo.IsDir() {
+		return m.renderDirectoryPreview(r, itemPath, previewHeight) + clearCmd
+	}
+
+	if isImageFile(itemPath) {
+		return m.renderImagePreview(box, itemPath, previewWidth, previewHeight, m.fullWidth-previewWidth+1)
+	}
+
+	return m.renderTextPreview(r, box, itemPath, previewWidth, previewHeight) + clearCmd
+}
+
 func getBatSyntaxHighlightedContent(itemPath string, previewLine int, background string) (string, error) {
 	// --plain: use the plain style without line numbers and decorations
 	// --force-colorization: force colorization for non-interactive shell output
